@@ -229,13 +229,21 @@ def config_handler(from_GUI = False, param_file_path = ''):
                     if len(custom_glycans) == 1:
                         custom_glycans = temp_custom_glycans_list.split("\n")
                     to_remove = []
+                    to_add = []
                     if len(custom_glycans) > 1:
                         for i_i, i in enumerate(custom_glycans):
                             custom_glycans[i_i] = i.strip()
-                            if len(i) == 0:
+                            if len(custom_glycans[i_i]) == 0:
                                 to_remove.append(i_i)
+                                continue
+                            if len(custom_glycans[i_i].split("/")) > 1:
+                                splitted_glycan = custom_glycans[i_i].split("/")
+                                custom_glycans[i_i] = splitted_glycan[0]
+                                to_add.append(splitted_glycan[1])
                     for i in sorted(to_remove, reverse = True):
                         del custom_glycans[i]
+                    for i in to_add:
+                        custom_glycans.append(i)
                 except:
                     print("Custom glycans file not found. Check the path to\nthe file you've inputted in the parameters file.")
                     print("Close the window or press CTRL+C to exit.")
@@ -246,10 +254,18 @@ def config_handler(from_GUI = False, param_file_path = ''):
                         os._exit(1)
             else:
                 custom_glycans = config['library_building_modes']['custom_glycans_list'].split(",")
+                to_add = []
                 for i_i, i in enumerate(custom_glycans):
                     custom_glycans[i_i] = i.strip()
-                    if len(i) == 0:
+                    if len(custom_glycans[i_i]) == 0:
                         custom_glycans = custom_glycans[:i_i]+custom_glycans[i_i+1:]
+                        continue
+                    if len(custom_glycans[i_i].split("/")) > 1:
+                        splitted_glycan = custom_glycans[i_i].split("/")
+                        custom_glycans[i_i] = splitted_glycan[0]
+                        to_add.append(splitted_glycan[1])
+                for i in to_add:
+                    custom_glycans.append(i)
             custom_glycans_list[1] = custom_glycans
     elif library_mode != 'custom_library' and library_mode != 'import_library' and library_mode != 'generate_library':
         print("Wrong mode selected. Use 'import_library',\n'custom_library' or 'generate_library' in mode\nunder 'library_building_modes', in the\nparameters file.")
@@ -371,7 +387,11 @@ def config_handler(from_GUI = False, param_file_path = ''):
                     reporter_ions = reporter_ions[:i_i]+reporter_ions[i_i+1:]
         align_chromatograms = config['post-analysis/reanalysis'].getboolean('align_chromatograms')
         percentage_auc = float(config['post-analysis/reanalysis']['auc_percentage_threshold'])/100
-        max_ppm = int(config['post-analysis/reanalysis']['max_ppm_threshold'])
+        ppm_setting = config['post-analysis/reanalysis']['max_ppm_threshold'].split(",")
+        if len(ppm_setting) > 1:
+            max_ppm = float(config['post-analysis/reanalysis']['max_ppm_threshold'])
+        else:
+            max_ppm = (float(ppm_setting[0]), float(ppm_setting[1]))
         iso_fit_score = float(config['post-analysis/reanalysis']['isotopic_fitting_score_threshold'])
         curve_fit_score = float(config['post-analysis/reanalysis']['curve_fitting_score_threshold'])
         s_to_n = float(config['post-analysis/reanalysis']['signal_to_noise_threshold'])
