@@ -144,7 +144,8 @@ def eic_from_glycan(files,
                     inf_arrays,
                     threads_arrays,
                     rt_arrays,
-                    ms1_id):
+                    ms1_id,
+                    rewind = 10):
     '''Generates a very processed EIC for each adduct of each glycan of each sample.
     Removes non-monoisotopic peaks, check charges and calculates multiple quality scoring
     data.
@@ -293,6 +294,33 @@ def eic_from_glycan(files,
                                  ms1_id[-1],
                                  adduct_mass,
                                  adduct_charge)
+                if len(buffer) > 4 and buffer[-1] != None and buffer[-2] != None and buffer[-3] != None and buffer[-4] == None:
+                    for l_l in range(-3, -3-rewind, -1):
+                        if l_l == -len(buffer) or buffer[l_l-1] != None: # the l_l limit is set to -10 to be compatible with buffer_size
+                            break
+                        analyze_mz_array(j[thread_numbers[k_k+l_l]]['m/z array'],
+                                         j[thread_numbers[k_k+l_l]]['intensity array'],
+                                         glycan_info,
+                                         tolerance,
+                                         min_isotops,
+                                         noise,
+                                         avg_noise,
+                                         max_charges,
+                                         ppm_info,
+                                         iso_fitting_quality,
+                                         data,
+                                         raw_data,
+                                         isotopic_fits,
+                                         i,
+                                         j_j,
+                                         thread_numbers[k_k+l_l],
+                                         j[thread_numbers[k_k+l_l]]['retentionTime'],
+                                         ms1_id[j_j][k_k+l_l],
+                                         ms1_id[-1],
+                                         adduct_mass,
+                                         adduct_charge,
+                                         retest = True,
+                                         retest_no = l_l-1)
     return data, ppm_info, iso_fitting_quality, verbose_info, raw_data, isotopic_fits
 
     
@@ -317,7 +345,9 @@ def analyze_mz_array(sliced_mz,
                      last_ms1_id,
                      adduct_mass,
                      adduct_charge,
-                     filtered = True):
+                     filtered = True,
+                     retest = False,
+                     retest_no = 0):
     '''The core function of eic_from_glycan. Analyzes a single spectrum and outputs
     relevant information, such as the PPM-error of the monoisotopic peak, isotopic fitting
     information, etc.
@@ -431,9 +461,10 @@ def analyze_mz_array(sliced_mz,
     if target_mz > sliced_mz[-1]:
         mz_id = -1
     else:
-        mz_id = General_Functions.binary_search_with_tolerance(sliced_mz, target_mz, 0, sliced_mz_length, General_Functions.tolerance_calc(tolerance[0], tolerance[1], target_mz))
+        mz_id = General_Functions.binary_search_with_tolerance(sliced_mz, target_mz, 0, sliced_mz_length, General_Functions.tolerance_calc(tolerance[0], tolerance[1], target_mz), sliced_int)
     
     if mz_id != -1 and sliced_int[mz_id] >= local_noise*0.5:
+        found_mz = sliced_mz[mz_id]
         charge_range = range(1, abs(max_charges)*2)
         intensity = sliced_int[mz_id] #variable to sum the total deisotopotized intensity
         mono_int = sliced_int[mz_id] #variable to sum the total intensity of the monoisotopic peak
@@ -443,30 +474,43 @@ def analyze_mz_array(sliced_mz,
         if filtered == True:
             iso_actual = [1]
             bad = False #here starts quality checks
-            for i in charge_range: #check if it's monoisotopic and correct charge
-                temp_id = General_Functions.binary_search_with_tolerance(sliced_mz, target_mz-(General_Functions.h_mass/i), 0, mz_id, General_Functions.tolerance_calc(tolerance[0], tolerance[1], target_mz-(General_Functions.h_mass/i)))
-                if temp_id != -1:
-                    expected_value = 1/((sliced_mz[temp_id]*i*0.0006)+0.1401) #based on linear regression of the relationship between masses and the second isotopic peak relative intensity of the average of different organic macromolecules
-                    if (sliced_int[temp_id] > mono_int*expected_value*0.6):
-                        bad = True
-                        break
-                if i == 1: #ignores charge 1 due to the fact that any charge distribution will find a hit on that one
-                    continue
-                temp_id = General_Functions.binary_search_with_tolerance(sliced_mz, target_mz+(General_Functions.h_mass/i), mz_id, sliced_mz_length, General_Functions.tolerance_calc(tolerance[0], tolerance[1], target_mz+(General_Functions.h_mass/i)))
-                if temp_id != -1:
-                    expected_value = (sliced_mz[temp_id]*i*0.0006)+0.1401 #based on linear regression of the relationship between masses and the second isotopic peak relative intensity of the average of different organic macromolecules
-                    if i != adduct_charge and (sliced_int[temp_id] > mono_int*expected_value*0.6):
-                        bad = True
-                        break
+            margin = 0.2 #0.2 = 20%, 0.4 = 40% - margin for checking, higher the margin, more strict is the checking and less glycans will probably be found
             
+            temp_id = General_Functions.binary_search_with_tolerance(sliced_mz, found_mz+(General_Functions.h_mass/abs(adduct_charge)), mz_id, sliced_mz_length, General_Functions.tolerance_calc(tolerance[0], tolerance[1], found_mz+(General_Functions.h_mass/abs(adduct_charge))), sliced_int) #check if second isotopic is actually present or not
+            if temp_id == -1:
+                bad = True
+                
+            if not bad:
+                for i in charge_range: #check if it's monoisotopic and correct charge
+                    if (len(buffer) <= 2 or (len(buffer) > 2 and buffer[-3] == None)) and not retest: #only check if it's monoisotopic if at least one of the last 3 RT got nothing...
+                        temp_id = General_Functions.binary_search_with_tolerance(sliced_mz, found_mz-(General_Functions.h_mass/i), 0, mz_id, General_Functions.tolerance_calc(tolerance[0], tolerance[1], found_mz-(General_Functions.h_mass/i)), sliced_int) #check monoisotopic
+                        if temp_id != -1 and sliced_int[temp_id] > 0:
+                            expected_value = (sliced_mz[temp_id]*i*0.0006)+0.1401 #based on linear regression of the relationship between masses and the second isotopic peak relative intensity of the average of different organic macromolecules
+                            # print(f"RT: {ret_time}, Check mono, expected value greater than: {expected_value*(1+(margin*2))}, theoretical mass of monoisotopic: {sliced_mz[temp_id]*i}, charges: {i}, second isotopic actual: {mono_int/sliced_int[temp_id]}")
+                            if (mono_int/sliced_int[temp_id] < expected_value*(1+(margin*2))):
+                                # print("bad")
+                                bad = True
+                                break
+                            # print("ok")                        
+                    if i == 1 or i == abs(adduct_charge) or (i == 2 and abs(adduct_charge) == 4) or (i == 3 and abs(adduct_charge) == 6): #ignores charge 1 due to the fact that any charge distribution will find a hit on that one
+                        continue
+                    temp_id = General_Functions.binary_search_with_tolerance(sliced_mz, found_mz+(General_Functions.h_mass/i), mz_id, sliced_mz_length, General_Functions.tolerance_calc(tolerance[0], tolerance[1], found_mz+(General_Functions.h_mass/i)), sliced_int) #check for correct charge
+                    if temp_id != -1:
+                        expected_value = (target_mz*i*0.0006)+0.1401 #based on linear regression of the relationship between masses and the second isotopic peak relative intensity of the average of different organic macromolecules
+                        # print(f"RT: {ret_time}, Check charge, expected value smaller than: {expected_value*(1-margin)}, theoretical mass of monoisotopic: {target_mz*i}, charges: {i}, second isotopic actual: {sliced_int[temp_id]/mono_int}")
+                        if (sliced_int[temp_id]/mono_int > expected_value*(1-margin)):
+                            # print("bad")
+                            bad = True
+                            break
+                        # print("ok")
             if not bad:
                 isos_found = 0
                 mz_isos = []
                 for i_i, i in enumerate(glycan_info['Isotopic_Distribution_Masses']): #check isotopic peaks and add to the intensity
                     if i_i == 0: #ignores monoisotopic this time around
                         continue
-                    temp_id = General_Functions.binary_search_with_tolerance(sliced_mz, (i+adduct_mass)/adduct_charge, mz_id, sliced_mz_length, General_Functions.tolerance_calc(tolerance[0], tolerance[1], (i+adduct_mass)/adduct_charge))
-                    if temp_id != -1:
+                    temp_id = General_Functions.binary_search_with_tolerance(sliced_mz, found_mz+(i_i*(General_Functions.h_mass/abs(adduct_charge))), mz_id, sliced_mz_length, General_Functions.tolerance_calc(tolerance[0], tolerance[1], found_mz+(i_i*(General_Functions.h_mass/abs(adduct_charge)))), sliced_int)
+                    if temp_id != -1 and sliced_int[temp_id] > 0:
                         isos_found += 1
                         mz_isos.append(sliced_mz[temp_id])
                         iso_actual.append(sliced_int[temp_id]/mono_int)
@@ -480,9 +524,18 @@ def analyze_mz_array(sliced_mz,
                             break
                         else:
                             break
+                            
+            if not bad and (iso_actual[1] < 0.2 or iso_actual[1] > 5): #this should avoid situations where it's obvious that it's picking the wrong charge because the second peak is almost invisible compared to the third and first, which when z=2 means that it's very likely actually a singly charge compound, for example
+                if len(iso_actual) > 2:
+                    if iso_actual[2] > iso_actual[1]*10 or iso_actual[1] > 5:
+                        bad = True
+                else:
+                    if iso_actual[1] < 0.2 or iso_actual[1] > 5: #smallest glycan should have the second iso_actual somewhere around 0.5, so a cutoff lower than that is fine
+                        bad = True
                         
             if bad:
-                buffer.append(None)
+                if not retest:
+                    buffer.append(None)
             else:
                 iso_target = glycan_info['Isotopic_Distribution'][:isos_found+1]
                 
@@ -493,7 +546,11 @@ def analyze_mz_array(sliced_mz,
                         continue
                     intensities = [i, iso_actual[i_i]]
                     ratio = min(intensities)/max(intensities)
-                    corrected_ratio = 1 - (1 - ratio) ** 1.35
+                    
+                    #scales the score in a sigmoid, with steepness determine by k_value
+                    k_value = 10
+                    corrected_ratio = 1 / (1 + numpy.exp(-k_value * (ratio - 0.5)))
+                    
                     ratios.append(corrected_ratio)
                     weights.append(1/(exp(1.25*i_i)))
                 
@@ -505,48 +562,54 @@ def analyze_mz_array(sliced_mz,
                 
                 mz_isos = [glycan_info['Adducts_mz'][glycan_id]]+mz_isos
                 
-                buffer.append(([glycan_id, file_id, ms1_id, float("%.4f" % round(ret_time, 4))], [ppm_error, iso_quali, intensity, [mz_isos, iso_target, iso_actual, iso_quali]]))
-        
-        #dynamical clean-up of buffer
-        min_in_a_row = 4
-        buffer_size = 10
-        if filtered and (len(buffer) >= buffer_size or ms1_id == last_ms1_id): 
-            highest_in_a_row = 0
-            in_a_row = 0
-            for i_i, i in enumerate(buffer):
-                if i != None:
-                    in_a_row += 1
-                    if in_a_row > highest_in_a_row:
-                        highest_in_a_row = in_a_row
+                if not retest:
+                    buffer.append(([glycan_id, file_id, ms1_id, float("%.4f" % round(ret_time, 4))], [ppm_error, iso_quali, intensity, [mz_isos, iso_target, iso_actual, iso_quali]]))
                 else:
-                    if in_a_row > 0 and in_a_row < min_in_a_row:
-                        for k_k, k in enumerate(buffer):
-                            if k_k >= i_i:
-                                break
-                            if k_k >= i_i-in_a_row:
-                                buffer[k_k] = None
-                    in_a_row = 0
-            if highest_in_a_row >= min_in_a_row:
-                for i in buffer:
-                    if i != None:
-                        ppm_info[i[0][0]][i[0][1]][i[0][2]] = i[1][0]
-                        iso_fitting_quality[i[0][0]][i[0][1]][i[0][2]] = i[1][1]
-                        data[i[0][0]][i[0][1]][1][i[0][2]] = i[1][2]
-                        isotopic_fits[i[0][0]][i[0][1]][i[0][3]] = i[1][3]
-            buffer.pop(0)
+                    buffer[retest_no] = ([glycan_id, file_id, ms1_id, float("%.4f" % round(ret_time, 4))], [ppm_error, iso_quali, intensity, [mz_isos, iso_target, iso_actual, iso_quali]])
                 
-        elif not filtered:
+    else:
+        if filtered == True:
+            if not retest:
+                buffer.append(None)
+        else:
+            info = ([glycan_id, file_id, ms1_id, float("%.4f" % round(ret_time, 4))], [inf, 1.0, 0.0, [[], [], [], 1.0]])
+            isotopic_fits[info[0][0]][info[0][1]][info[0][3]] = info[1][3]
+    
+    # print(f"Buffer before clean-up: {buffer}")
+    
+    #dynamical clean-up of buffer
+    min_in_a_row = 4
+    buffer_size = 10
+    if filtered:
+        in_a_row = 0
+        for i_i, i in enumerate(buffer):
+            if i != None:
+                in_a_row += 1
+            else:
+                in_a_row = 0
+                for k_k, k in enumerate(buffer): #this clears the buffer up to the point where it was checked, only once "None" is found
+                    if k_k >= i_i:
+                        break
+                    buffer[k_k] = None
+        if in_a_row >= min_in_a_row:
+            for i in buffer:
+                if i != None:
+                    ppm_info[i[0][0]][i[0][1]][i[0][2]] = i[1][0]
+                    iso_fitting_quality[i[0][0]][i[0][1]][i[0][2]] = i[1][1]
+                    data[i[0][0]][i[0][1]][1][i[0][2]] = i[1][2]
+                    isotopic_fits[i[0][0]][i[0][1]][i[0][3]] = i[1][3]
+        if len(buffer) >= buffer_size: #this means that the buffer will be worked on once it gets to the buffer_size or over it or the end of the MS1 array is reached
+            buffer.pop(0)
+            
+    elif not filtered:
+        if not retest:
             info = ([glycan_id, file_id, ms1_id, float("%.4f" % round(ret_time, 4))], [ppm_error, 1.0, mono_int, [[], [], [], 1.0]])
             ppm_info[info[0][0]][info[0][1]][info[0][2]] = info[1][0]
             iso_fitting_quality[info[0][0]][info[0][1]][info[0][2]] = info[1][1]
             data[info[0][0]][info[0][1]][1][info[0][2]] = info[1][2]
             isotopic_fits[info[0][0]][info[0][1]][info[0][3]] = info[1][3]
-    else:
-        if filtered == True:
-            buffer.append(None)
-        else:
-            info = ([glycan_id, file_id, ms1_id, float("%.4f" % round(ret_time, 4))], [inf, 1.0, 0.0, [[], [], [], 1.0]])
-            isotopic_fits[info[0][0]][info[0][1]][info[0][3]] = info[1][3]
+        
+    # print(f"Buffer after clean-up: {buffer}")
     
 def eic_smoothing(y, lmbd = 100, d = 2):
     '''Implementation of the Whittaker smoothing algorithm,
@@ -688,7 +751,8 @@ def iso_fit_score_calc(iso_fits,
         Average of the isotopic fits score calculated for the interval of
         the peak of the glycan, weighted by the gaussian fit of it.
     '''
-    iso_fit_score = numpy.average(iso_fits[peak['peak_interval_id'][0]:peak['peak_interval_id'][1]+1], weights = weights_list)
+    new_weights = [i**2 for i in weights_list]
+    iso_fit_score = numpy.average(iso_fits[peak['peak_interval_id'][0]:peak['peak_interval_id'][1]+1], weights = new_weights)
     return iso_fit_score
     
 def average_ppm_calc(ppm_array,
@@ -725,6 +789,7 @@ def average_ppm_calc(ppm_array,
         The number of missing points in the peak has their ppm difference set to the
         tolerance of the analysis.
     '''
+    new_weights = [i**2 for i in weights_list]
     ppms = []
     ppm_default = General_Functions.calculate_ppm_diff(tolerance[2]-General_Functions.tolerance_calc(tolerance[0], tolerance[1], tolerance[2]), tolerance[2])
     missing_points = 0
@@ -734,11 +799,12 @@ def average_ppm_calc(ppm_array,
         else:
             ppms.append(ppm_default)
             missing_points+= 1
-    regular_mean = numpy.average(ppms, weights = weights_list)
+    regular_mean = numpy.average(ppms, weights = new_weights)
     return regular_mean, missing_points
 
 def peaks_from_eic(rt_int, 
                    rt_int_smoothed,
+                   raw_rt_int,
                    rt_interval,
                    min_ppp,
                    close_peaks,
@@ -782,30 +848,20 @@ def peaks_from_eic(rt_int,
     peaks_ranges = []
     datapoints_per_time = int((0.2/(rt_int[0][rt_int[1].index(max(rt_int[1]))]-rt_int[0][rt_int[1].index(max(rt_int[1]))-1]))*(rt_int[0][-1]/60))
     maximums_index = []
-    array_max = max(rt_int_smoothed[1])
     min_relative_int_peak = 0.001
-    cutoff = array_max
-    while cutoff > min_relative_int_peak*max(rt_int_smoothed[1]):
-        for i_i, i in enumerate(rt_int_smoothed[1]):
-            if (rt_int[0][i_i] >= rt_interval[1] or rt_int[0][i_i] == rt_int[0][-2]):
-                break    
-            if rt_int[0][i_i] >= rt_interval[0]:
-                if i >= cutoff:
-                    if i_i not in maximums_index:
-                        if rt_int_smoothed[1][i_i-1] <= i and rt_int_smoothed[1][i_i+1] <= i:
-                            found = False
-                            for k in maximums_index:
-                                if abs(k-i_i) < datapoints_per_time:
-                                    found = True
-                                    break
-                            if not found:
-                                maximums_index.append(i_i)
-        cutoff = cutoff-(array_max*min_relative_int_peak)
-    
-    # print(datapoints_per_time, sorted(maximums_index))
+    threshold = min_relative_int_peak*max(rt_int_smoothed[1])
+    for i_i, i in enumerate(rt_int_smoothed[1]):
+        if (rt_int[0][i_i] > rt_interval[1] or rt_int[0][i_i] == rt_int[0][-2]):
+            break    
+        if rt_int[0][i_i] >= rt_interval[0]:
+            if i > threshold and rt_int_smoothed[1][i_i-1] <= i and rt_int_smoothed[1][i_i+1] <= i:
+                if len(maximums_index) == 0:
+                    maximums_index.append(i_i)
+                elif len(maximums_index) > 0 and i_i-maximums_index[-1] >= datapoints_per_time:
+                    maximums_index.append(i_i)
     
     former_peak_limit = 0
-    for i_i, i in enumerate(sorted(maximums_index)):
+    for i_i, i in enumerate(maximums_index):
         #print('starting id: ', i, 'former peak limit: ', former_peak_limit)
         peak_limits = []
         temp_min = inf
@@ -846,30 +902,32 @@ def peaks_from_eic(rt_int,
         if len(peak_limits) == 2:
             peaks_ranges = peaks_ranges + peak_limits
             
-    # print(peaks_ranges)
+    #print(peaks_ranges)
     
     removal = []        
     for i_i, i in enumerate(peaks_ranges):
-        if i_i > 1:
+        if i_i > 1 and i != peaks_ranges[-1]:
             #print(peaks_ranges[i_i-1], peaks_ranges[i_i])
             if peaks_ranges[i_i] == peaks_ranges[i_i-1]:
                 #print("True!")
-                if (rt_int_smoothed[1][i] > max(rt_int_smoothed[1][peaks_ranges[i_i-2]:peaks_ranges[i_i+1]])*0.8 or 
-                    abs(max(rt_int_smoothed[1][peaks_ranges[i_i-2]:peaks_ranges[i_i-1]])-rt_int_smoothed[1][peaks_ranges[i_i]]) < rt_int_smoothed[1][peaks_ranges[i_i]]*0.1 or 
-                    abs(max(rt_int_smoothed[1][peaks_ranges[i_i]:peaks_ranges[i_i+1]])-rt_int_smoothed[1][peaks_ranges[i_i]]) < rt_int_smoothed[1][peaks_ranges[i_i]]*0.1):
+                if (rt_int_smoothed[1][i] > max(rt_int_smoothed[1][peaks_ranges[i_i-2]:peaks_ranges[i_i+1]])*0.8 
+                    or abs(max(rt_int_smoothed[1][peaks_ranges[i_i-2]:peaks_ranges[i_i-1]])-rt_int_smoothed[1][peaks_ranges[i_i]]) < rt_int_smoothed[1][peaks_ranges[i_i]]*0.1 
+                    or abs(max(rt_int_smoothed[1][peaks_ranges[i_i]:peaks_ranges[i_i+1]])-rt_int_smoothed[1][peaks_ranges[i_i]]) < rt_int_smoothed[1][peaks_ranges[i_i]]*0.1):
                     
                     removal.append(i_i-1)
                     removal.append(i_i)
     for i in sorted(removal, reverse=True):
         del peaks_ranges[i]
         
-    # print(peaks_ranges)
+    #print(peaks_ranges)
     
     for i_i, i in enumerate(peaks_ranges):
         if i_i % 2 == 0:
             peak_limits = [i, peaks_ranges[i_i+1]]
+            if peak_limits[0] == peak_limits[1]:
+                continue
             temp_peak_width = (rt_int[0][peak_limits[1]]-rt_int[0][peak_limits[0]])
-            peaks.append({'id': i, 'rt': rt_int[0][rt_int_smoothed[1].index(max(rt_int_smoothed[1][peak_limits[0]:peak_limits[1]]))], 'int': max(rt_int_smoothed[1][peak_limits[0]:peak_limits[1]]), 'peak_width': temp_peak_width, 'peak_interval': (rt_int[0][peak_limits[0]], rt_int[0][peak_limits[1]]), 'peak_interval_id': (peak_limits[0], peak_limits[1])})
+            peaks.append({'id': i, 'rt': rt_int[0][rt_int_smoothed[1].index(max(rt_int_smoothed[1][peak_limits[0]:peak_limits[1]+1]))], 'int': max(raw_rt_int[1][peak_limits[0]:peak_limits[1]+1]), 'peak_width': temp_peak_width, 'peak_interval': (rt_int[0][peak_limits[0]], rt_int[0][peak_limits[1]]), 'peak_interval_id': (peak_limits[0], peak_limits[1])})
     
     #print(peaks)
         
