@@ -44,7 +44,9 @@ monosaccharides = {
     "Am": ("Lactonized Acetyl Neuraminic Acid alpha2,3 bound", "C11O8N2H20", {"C": 11, "O": 7, "N": 2, "H": 18}),
     "E": ("Ethyl-Esterified Acetyl Neuraminic Acid alpha2,6 bound", "C13O9NH23", {"C": 13, "O": 8, "N": 1, "H": 21}),
     "F": ("Fucose", "C6O5H12", {"C": 6, "O": 4, "N": 0, "H": 10}),
-    "G": ("Glycolyl Neuraminic Acid", "C11O10NH19", {"C": 11, "O": 9, "N": 1, "H": 17})
+    "G": ("Glycolyl Neuraminic Acid", "C11O10NH19", {"C": 11, "O": 9, "N": 1, "H": 17}),
+    "AmG": ("Lactonized Glycolyl Neuraminic Acid alpha2,3 bound", "C11O8N2H20", {"C": 11, "O": 8, "N": 2, "H": 18}),
+    "EG": ("Ethyl-Esterified Glycolyl Neuraminic Acid alpha2,6 bound", "C13O9NH23", {"C": 13, "O": 9, "N": 1, "H": 21})
     }
 '''A hardcoded dictionary containing each single letter code for monosaccharides as key
 and a tuple containing the full monosaccharide name, its full molecular formula and its
@@ -60,7 +62,7 @@ times during a run.
 ##General functions (these functions use only external libraries, such as itertools and
 ##pyteomics).
 
-def binary_search_with_tolerance(arr, target, low, high, tolerance):
+def binary_search_with_tolerance(arr, target, low, high, tolerance, int_arr = []):
     '''
     '''
     # Base case: if the range is invalid, the target is not in the array
@@ -72,27 +74,34 @@ def binary_search_with_tolerance(arr, target, low, high, tolerance):
     
     # Check if the target is within the tolerance range of the middle element
     if abs(arr[mid] - target) <= tolerance:
-        mid_distance = abs(arr[mid] - target)
-        smallest_distance = mid_distance
-        smallest_distance_id = mid
-        if mid != len(arr)-1 and arr[mid+1] <= tolerance:
-            mid_plus_distance = abs(arr[mid+1] - target)
-            if mid_plus_distance < smallest_distance:
-                smallest_distance = mid_plus_distance
-                smallest_distance_id = mid+1
-        if mid != 0 and arr[mid-1] <= tolerance:
-            mid_minus_distance = abs(arr[mid-1] - target)
-            if mid_minus_distance < smallest_distance:
-                smallest_distance = mid_minus_distance
-                smallest_distance_id = mid-1
-                
-        return smallest_distance_id  # Closest target found within tolerance at or around index mid
+        range_width = 5
+        range_search = [mid, mid+1]
+        for i in range(mid, mid-range_width, -1):
+            if i == 0 or arr[i] < target-tolerance:
+                break
+            range_search[0] = i
+        for i in range(mid+1, mid+range_width+1):
+            range_search[1] = i
+            if i > high or arr[i] > target+tolerance:
+                break
+        if len(int_arr) != 0:
+            array_slice = int_arr[range_search[0]:range_search[1]]
+        else:
+            array_slice = arr[range_search[0]:range_search[1]]
+        if len(array_slice) == 0:
+            return -1
+        if len(int_arr) != 0:
+            selected_id = range_search[0]+numpy.argmax(array_slice)
+        else:
+            selected_id = range_search[0]+(numpy.abs(array_slice - target).argmin())
+        
+        return selected_id
     elif arr[mid] < target:
         # If target is greater, ignore the left half
-        return binary_search_with_tolerance(arr, target, mid + 1, high, tolerance)
+        return binary_search_with_tolerance(arr, target, mid + 1, high, tolerance, int_arr)
     else:
         # If target is smaller, ignore the right half
-        return binary_search_with_tolerance(arr, target, low, mid - 1, tolerance)
+        return binary_search_with_tolerance(arr, target, low, mid - 1, tolerance, int_arr)
 
 def linear_regression(x, y, th = 2.5):
     '''Traces a linear regression of supplied 2d data points and returns the slope,
@@ -192,7 +201,7 @@ def make_gg(temp_dir, save_dir, filename):
         Creates filename.gg file, containing files in temp_dir, at the save_dir.
     '''
     files_list = os.listdir(temp_dir)
-    with zipfile.ZipFile(save_dir+filename+".gg", 'w') as zipf:
+    with zipfile.ZipFile(save_dir+filename+".gg", 'w', compression=zipfile.ZIP_DEFLATED) as zipf:
         for file in files_list:
             zipf.write(temp_dir+"/"+str(file), arcname=file)
             
@@ -593,7 +602,11 @@ def count_seq_letters(string):
             count = string.count(i)
             if i == "L":
                 friendly_letters['Am'] = count
-            elif i != "L":
+            if i == "A":
+                friendly_letters['AmG'] = count
+            if i == "R":
+                friendly_letters['EG'] = count
+            else:
                 friendly_letters[i] = count
     return friendly_letters
 
@@ -635,7 +648,7 @@ def sum_monos(*compositions):
     summed_comp : dict
         Dictionary containing the sum of each monosaccharides of the compositions.
     '''
-    summed_comp = {"H": 0, "N": 0, "X": 0, "S": 0, "Am": 0, "E": 0, "F": 0, "G": 0, "T": 0}
+    summed_comp = {"H": 0, "N": 0, "X": 0, "S": 0, "Am": 0, "E": 0, "F": 0, "G": 0, "AmG": 0, "EG": 0, "T": 0}
     for i in compositions:
         for j in i:
             summed_comp[j]+=i[j]
